;+
;=========================================================================================
; This file contains an implementation of a broadband beam profile model 
; of the three photometric channels of the Herschel-SPIRE instrument.
; The implementation follows Griffin, North, Schulz et al. 2013, MNRAS 434, 992-1004
; except for a tweak in the effective frequencies to achieve better agreement with the
; measured solid angles.
;
; This file contains the following IDL procedures and functions:
;
; pro 		hs_broadbpsf, det, str, fstr, alpha, nu, transm, bbpsf, nu_eff=nu_eff
; function 	hs_monocrpsf, det, str, fstr, nu, nu_eff
; pro 		hs_rdfilt, fstr
; pro 		hs_rdradpsf, str
; pro 		test_hs_broadbpsf
;
; In addition the following files are needed: 
;
; SpirePhotRSRF.fits
;
; PLW_MCore_10.csv    
; PMW_MCore_10.csv                   
; PSW_MCore_10.csv
;
; The files either came with this file or can be found online.
;
; The FITS file is at:
; https://nhscsci.ipac.caltech.edu//spire/docs/SpirePhotRSRF.fits
; The other files can be obtained from the SPIRE public webpage at: 
; http://herschel.esac.esa.int/twiki/bin/view/Public/SpirePhotometerBeamProfile
; or from
; https://nhscsci.ipac.caltech.edu/sc/index.php/Spire/PhotBeamProfileDataAndAnalysis
;
; Their absolute path must be configured at the end of this file in the 
; as system variable !hs_datapath'
;
; To test the installation in IDL first compile the file with:
; 	IDL> .run hs_beamprofilemodel.pro
; then execute the test procedure:
; 	IDL> test_hs_broadbpsf                 
; 	          SED     PSW    PMW    PLW
; 	      Neptune    454.   803.  1687.
; 	 alpha=-1 SED    469.   833.  1803.
; 	IDL>
; The result should look as above.
;
;
; Author: Bernhard Schulz (SPIRE-ICC, NHSC)
;
;=========================================================================================
;-


;+
;=========================================================================================
; Read SPIRE filter profiles into structure
;
; Input:
;       fstr        named variable
;
; Output:
;       fstr        this variable will contain an output structure containing
;                   instrumental relative response functions for point sources and 
;                   aperture efficiency tables, both frequency dependent.
;                   The frequency is in units of Hz.
;
;    FREQ            DOUBLE    Array[512]
;    FPSW            DOUBLE    Array[512]
;    FPMW            DOUBLE    Array[512]
;    FPLW            DOUBLE    Array[512]
;    nu_ref_psw      DOUBLE    double       reference frequency PSW
;    nu_ref_pmw      DOUBLE    double       reference frequency PMW
;    nu_ref_plw      DOUBLE    double       reference frequency PLW
;
;
; Author: Bernhard Schulz
;
; Edition History
; 2013-07-18 B. Schulz initial test version
; 2013-10-28 B. Schulz added header and changed file path
; 2013-10-29 B. Schulz renamed to hs_rdfilt and changes in comments
; 2014-10-10 B. Schulz added filter reference wavelengths
; 2015-03-19 B. Schulz commented out table printing and added messages about written FITS files
; 2015-10-08 B. Schulz updates for Aug 2015 (HIPE 14) update of PSF calibration
;
;=========================================================================================
;-

pro hs_rdfilt, fstr

    hs_getunits, unit, const

    ; Read filter profiles
    fxbopen, un, !hs_datapath+'SpirePhotRSRF.fits', 1, hd
    fxbread, un, wav, 'wavenumber'
    fxbread, un, fpsw, 'psw'
    fxbread, un, fpmw, 'pmw'
    fxbread, un, fplw, 'plw'
    fxbclose, un

    freq = const.c/(10000./wav*unit.micron)


    fstr = {freq:freq, $
           fpsw:fpsw, $ 
           fpmw:fpmw, $ 
           fplw:fplw, $
           nu_ref_psw: const.c/(250.0*unit.micron), $
           nu_ref_pmw: const.c/(350.0*unit.micron), $
           nu_ref_plw: const.c/(500.0*unit.micron)  $
           } 


end


;+
;=========================================================================================
; Read radial PSF data into structure
;
; Input:
;       str     named variable
;
; Output:
;       str     this variable will contain an output structure containing
;               core circular PSF data as described in 
;               North, Griffin, Amaral-Rogers, v2.6, Dec 2011 and 
;               The constant part was found to be unnecessary.
;
;    PSWCORE         FLOAT     Array[]
;    PMWCORE         FLOAT     Array[]
;    PLWCORE         FLOAT     Array[]
;    nu_eff_psw      DOUBLE    effective frequency of PSW psf
;    nu_eff_pmw      DOUBLE    effective frequency of PMW psf
;    nu_eff_plw      DOUBLE    effective frequency of PLW psf
;    gamma_psw       DOUBLE    exponent governing wavelength dep. of PSW FWHM
;    gamma_pmw       DOUBLE    exponent governing wavelength dep. of PMW FWHM
;    gamma_plw       DOUBLE    exponent governing wavelength dep. of PLW FWHM
;
; Author: Bernhard Schulz
;
; Edition History
; 2012-01-16 B. Schulz initial test version
; 2012-08-23 B. Schulz added format and silent keywords
; 2013-07-11 B. Schulz change for official files
; 2013-10-29 B. Schulz name change to read_radpsf
; 2013-11-01 B. Schulz changed to read SCalPhotRadialCorrBeam_v1.fits
; 2014-10-09 B. Schulz removed constant part of beam profile and reinstated old file 
;                      reading method
; 2014-10-10 B. Schulz moved nu_eff and gamma from hs_monocrpsf into this structure and
;                      added hs_getunits
; 2015-10-08 B. Schulz moved updated effective frequencies based on Aug-2015 PSF update
;                      for HIPE 14.
;
;=========================================================================================
;-

pro hs_rdradpsf, str

    hs_getunits, unit, const

    readcol, !hs_datapath+"PLW_MCore_10.csv",  plwcore, format='D', /silent
    readcol, !hs_datapath+"PMW_MCore_10.csv",  pmwcore, format='D', /silent
    readcol, !hs_datapath+"PSW_MCore_10.csv",  pswcore, format='D', /silent

    ;fxbopen, un, !hs_datapath+'SCalPhotRadialCorrBeam_v1.fits', 'core', hd
    ;fxbread, un, rad2, 'radius' 
    ;fxbread, un, pswcore, 'PSW' 
    ;fxbread, un, pmwcore, 'PMW' 
    ;fxbread, un, plwcore, 'PLW' 
    ;fxbclose, un


    str = {pswcore:     pswcore, $ 
           pmwcore:     pmwcore, $ 
           plwcore:     plwcore, $
           nu_eff_psw:  1222.1457*unit.GHz, $   ;effective frequency of psf (new 2015-10-08)
           ;nu_eff_psw:  1224.0683*unit.GHz, $   ;effective frequency of psf (new 2014-10-09)
           ;nu_eff_psw: 1217.27*unit.GHz, $    ;effective frequency of psf (Chis' value 2013-11-01)
           ;nu_eff_psw: 1215.29*unit.GHz, $    ;effective frequency of psf (new 2013-07-22)
           nu_eff_pmw:   871.24726*unit.GHz, $ ;effective frequency of psf (new 2015-10-08)
           ;nu_eff_pmw:   873.06788*unit.GHz, $ ;effective frequency of psf (new 2014-10-09)
           ;nu_eff_pmw:  867.75*unit.GHz, $    ;effective frequency of psf (Chis' value 2013-11-01)
           ;nu_eff_pmw:  865.52*unit.GHz, $    ;effective frequency of psf (new 2013-07-22)
           nu_eff_plw:   612.35883*unit.GHz, $ ;effective frequency of psf (new 2015-10-08)
           ;nu_eff_plw:   609.86168*unit.GHz, $ ;effective frequency of psf (new 2014-10-09)
           ;nu_eff_plw:  610.87*unit.GHz, $    ;effective frequency of psf (Chis' value 2013-11-01)
           ;nu_eff_plw:  610.26*unit.GHz, $    ;effective frequency of psf (new 2013-07-22)
           gamma_psw:    -0.85, $              ;exponent governing wavelength dep. of FWHM
           gamma_pmw:    -0.85, $              ;exponent governing wavelength dep. of FWHM
           gamma_plw:    -0.85  $              ;exponent governing wavelength dep. of FWHM
          }
end



;+
;=========================================================================================
; Define structures of units and physical constants
;
; Input:
;   none
;
; Output:
;   unit    (structure)
;       unit.Jansky
;       unit.MJansky
;       unit.deg
;       unit.arcmin
;       unit.arcsec
;       unit.mm
;       unit.cm
;       unit.micron
;       unit.MHz
;       unit.GHz
;       unit.THz
;
;   const (structure)
;       const.c
;       const.au
;       const.h
;       const.k
;
; Units are SI units, angles are radians
;
; Author: B. Schulz
;
; Edition History
; 2008/02/03 B.Schulz initial test version
; 2011/11/08 B.Schulz added constants and units
; 2013/07/17 B.Schulz terahertz added
; 2013/10/29 B.Schulz renamed to hs_getunits and comments edited
;
;=========================================================================================
;
;-

pro hs_getunits, unit, const

  unit = { $
    Jansky  :1d-26,             $   ;W*m^-2*Hz^-1
    MJansky :1d-20,             $   ;W*m^-2*Hz^-1
    deg     :!PI/180.d,         $   ;radians
    arcmin  :!PI/180.d/60.d,    $   ;radians
    arcsec  :!PI/180.d/3600.d,  $   ;radians
    mm      :1d-3,              $   ;meter
    cm      :1d-2,              $   ;meter
    micron  :1d-6,              $   ;meter
    MHz     :1d6,               $   ;Megahertz
    GHz     :1d9,               $   ;Gigahertz
    THz     :1d12               $   ;Terahertz
    }
    

  const = { $
    c       :299792458.,        $   ;m/sec
    AU      :1.4959787069D11,   $   ;m
    h       :6.626D-34,         $   ;Joule*sec
    k       :1.38D-23           $   ;Joule/K
    }

return
end



;+
;=========================================================================================
; Obtain monochromatic radial SPIRE Photometer PSF
;
;
; Input:
;    det     (string) detector array name, one of ["PSW","PMW","PLW"]
;    str     this variable contains a structure as generated by the procedure 
;            hs_rdradpsf with core circular PSF data as described in 
;            North, Griffin, Amaral-Rogers, v2.6, Dec 2011 and
;            Griffin, North, Schulz et al. 2013.
;
;            PSWCORE         FLOAT     Array[501]
;            PMWCORE         FLOAT     Array[501]
;            PLWCORE         FLOAT     Array[501]
;            PSWCORE         FLOAT     Array[]
;            PMWCORE         FLOAT     Array[]
;            PLWCORE         FLOAT     Array[]
;            nu_eff_psw      DOUBLE    effective frequency of PSW psf
;            nu_eff_pmw      DOUBLE    effective frequency of PMW psf
;            nu_eff_plw      DOUBLE    effective frequency of PLW psf
;            gamma_psw       DOUBLE    exponent governing wavelength dep. of PSW FWHM
;            gamma_pmw       DOUBLE    exponent governing wavelength dep. of PMW FWHM
;            gamma_plw       DOUBLE    exponent governing wavelength dep. of PLW FWHM
;
;
;    fstr    this variable will contain an output structure containing
;            instrumental relative response functions for point sources and 
;            aperture efficiency tables, both frequency dependent.
;            The frequency is in units of Hz.
;
;            FREQ            DOUBLE    Array[512]
;            FPSW            DOUBLE    Array[512]
;            FPMW            DOUBLE    Array[512]
;            FPLW            DOUBLE    Array[512]
;            nu_ref_psw      DOUBLE    double       reference frequency PSW
;            nu_ref_pmw      DOUBLE    double       reference frequency PMW
;            nu_ref_plw      DOUBLE    double       reference frequency PLW
;
; Output:
;       (double array) function, 1400 element vector with normalized radial PSF
;       intensities from 0 to 1399 arcsec startig with radius 0
;  
; Optional:
;       nu      (double) input, frequency of the returned PSF 
;       nu_eff optional effective nu for PSF
;              for default see table in hs_monocrpsf()
;
; Output:
;
; Author: Bernhard Schulz
;
; Edition History
; 2012-01-16 B. Schulz initial test version
; 2012-02-09 B. Schulz update of gamma from 0.65 to 0.9, 0.8, 0.8 for PSW, PMW, PLW respectively
; 2013-07-11 B. Schulz update of gamma to 0.85, 0.85, 0.85 for PSW, PMW, PLW respectively
; 2013-07-17 B. Schulz corelim changed from (339, 256, 261) to (400, 400, 400)
; 2013-07-21 B. Schulz nu_eff change from (1223.16, 872.52, 614.48) to (1235.48, 882.27, 623.75)
;                      changed sign of gamma to negative as in paper and reversed scaling, 
;                      added nu_eff optional parameter, new determined effective frequencies 
;                      by omega comparison
; 2013-07-22 B. Schulz Re-calculated nu_eff from paper and inserted these, they are almost 
;                      correct but re-instated new frequencies (1215.29, 865.52, 610.26)
;                      calculated without aperture correction.
;                      They are slightly different from the ones with aperture correction.
; 2013-10-29 B. Schulz name change to hs_monocrpsf and comment changes
; 2014-10-09 B. Schulz removed constant component, removed CORELIM
; 2014-10-10 B. Schulz moved constants to structures str and fstr and removed hs_getunits
;                      added fstr to API
;=========================================================================================
;-

function hs_monocrpsf, det, str, fstr, nu, nu_eff

    case strupcase(det) of
      "PSW": begin
         psfcore = str.pswcore        ;frequency dependent part
         nu_ref = fstr.nu_ref_psw     ;reference frequency of filter
         if n_params() LE 4 then $
           nu_eff =  str.nu_eff_psw   ;effective frequency of psf
         gamma = str.gamma_psw        ;exponent governing wavelength dep. of FWHM
         end
      "PMW": begin
         psfcore = str.pmwcore        ;frequency dependent part
         nu_ref = fstr.nu_ref_pmw     ;reference frequency of filter
         if n_params() LE 4 then $
           nu_eff =  str.nu_eff_pmw   ;effective frequency of psf
         gamma = str.gamma_pmw        ;exponent governing wavelength dep. of FWHM
         end
      "PLW": begin
         psfcore = str.plwcore      ;frequency dependent part
         nu_ref = fstr.nu_ref_plw
         if n_params() LE 4 then $
           nu_eff =  str.nu_eff_plw   ;effective frequency of psf
         gamma = str.gamma_plw        ;exponent governing wavelength dep. of FWHM
         end
      else:  message, "Wrong detector name!"
    endcase

    theta = dindgen(n_elements(psfcore))

    if n_params() EQ 2 then nu = nu_ref

    psfcore = interpol(psfcore,theta, theta / (nu/nu_eff)^gamma )
    
return, psfcore

end



;+
;=========================================================================================
; Obtain wavelength dependent circular SPIRE Photometer PSW
; integrated over instrument transmission profile
;
; Sum up transmission profile times source SED weighted beam profiles for all frequencies
;
; Input:
;    det     (string) detector array name, one of ["PSW","PMW","PLW"]
;    str     this variable contains a structure as generated by the procedure 
;            hs_rdradpsf with core circular PSF data as described in 
;            North, Griffin, Amaral-Rogers, v2.6, Dec 2011 and
;            Griffin, North, Schulz et al. 2013.
;
;            PSWCORE         FLOAT     Array[501]
;            PMWCORE         FLOAT     Array[501]
;            PLWCORE         FLOAT     Array[501]
;            PSWCORE         FLOAT     Array[]
;            PMWCORE         FLOAT     Array[]
;            PLWCORE         FLOAT     Array[]
;            nu_eff_psw      DOUBLE    effective frequency of PSW psf
;            nu_eff_pmw      DOUBLE    effective frequency of PMW psf
;            nu_eff_plw      DOUBLE    effective frequency of PLW psf
;            gamma_psw       DOUBLE    exponent governing wavelength dep. of PSW FWHM
;            gamma_pmw       DOUBLE    exponent governing wavelength dep. of PMW FWHM
;            gamma_plw       DOUBLE    exponent governing wavelength dep. of PLW FWHM
;
;
;    fstr    this variable will contain an output structure containing
;            instrumental relative response functions for point sources and 
;            aperture efficiency tables, both frequency dependent.
;            The frequency is in units of Hz.
;
;            FREQ            DOUBLE    Array[512]
;            FPSW            DOUBLE    Array[512]
;            FPMW            DOUBLE    Array[512]
;            FPLW            DOUBLE    Array[512]
;            nu_ref_psw      DOUBLE    double       reference frequency PSW
;            nu_ref_pmw      DOUBLE    double       reference frequency PMW
;            nu_ref_plw      DOUBLE    double       reference frequency PLW
;
;       alpha   index of powerlaw source spectrum F_nu = c*nu^alpha where c is an a\
;               arbitrary constant and nu is frequency.
;  
;       nu_eff optional effective nu for PSF
;       for default see table in hs_monocrpsf()
;
; Output:
;       nu      array of frequencies in [Hz]
;       transm  transmission spectrum for selected detector array
;       bbpsf   intergrated broad band beam profile over wavelengths
;  
; Optional:
;       nu_eff  (double) input, frequency of the returned PSF 
;
; Output:
;
; Author: Bernhard Schulz
;
; Edition History
; 2013-07-21 B. Schulz initial test version
; 2013-07-22 B. Schulz removed aperture efficiency (C. North)
; 2013-10-29 B. Schulz name change to hs_broadbpsf
; 2014-10-09 B. Schulz removed constant part
; 2014-10-10 B. Schulz normalize beam profile peak to reference spectrum point source
;
;=========================================================================================
;-

pro hs_broadbpsf, det, str, fstr, alpha, nu, transm, bbpsf, nu_eff=nu_eff

    case strupcase(det) of 
      'PSW': begin
          transm = fstr.fpsw
          nu_ref = fstr.nu_ref_psw
        end
      'PMW': begin
          transm = fstr.fpmw
          nu_ref = fstr.nu_ref_pmw
        end
      'PLW': begin
          transm = fstr.fplw
          nu_ref = fstr.nu_ref_plw
        end
      else: message, 'No valid filter!'
    endcase

    nrad = n_elements(str.pswcore)

    nu = fstr.freq
    nnu = n_elements(nu)
 
    bbpsf = dblarr(nrad)
    for i=0, nnu-1 do begin
      if n_elements(nu_eff) EQ 0 then $
        bbpsf = bbpsf + $
                hs_monocrpsf(det, str, fstr, nu[i]) * transm[i] * (nu[i]/nu_ref)^alpha $
      else $
        bbpsf = bbpsf + $
                hs_monocrpsf(det, str, fstr, nu[i], nu_eff) * transm[i] * (nu[i]/nu_ref)^alpha
    endfor


    bbpsf = bbpsf / total(transm * (nu/nu_ref)^(-1))    ;normalize to standard source/spectrum

    return
end


;+
;=========================================================================================
; 
; Input:
;    det     (string) detector array name, one of ["PSW","PMW","PLW"]
;    str     this variable contains a structure as generated by the procedure 
;            hs_rdradpsf with core circular PSF data as described in 
;            North, Griffin, Amaral-Rogers, v2.6, Dec 2011 and
;            Griffin, North, Schulz et al. 2013.
;
;            PSWCORE         FLOAT     Array[501]
;            PMWCORE         FLOAT     Array[501]
;            PLWCORE         FLOAT     Array[501]
;            PSWCORE         FLOAT     Array[]
;            PMWCORE         FLOAT     Array[]
;            PLWCORE         FLOAT     Array[]
;            nu_eff_psw      DOUBLE    effective frequency of PSW psf
;            nu_eff_pmw      DOUBLE    effective frequency of PMW psf
;            nu_eff_plw      DOUBLE    effective frequency of PLW psf
;            gamma_psw       DOUBLE    exponent governing wavelength dep. of PSW FWHM
;            gamma_pmw       DOUBLE    exponent governing wavelength dep. of PMW FWHM
;            gamma_plw       DOUBLE    exponent governing wavelength dep. of PLW FWHM
;
;
;    fstr    this variable will contain an output structure containing
;            instrumental relative response functions for point sources and 
;            aperture efficiency tables, both frequency dependent.
;            The frequency is in units of Hz.
;
;            FREQ            DOUBLE    Array[512]
;            FPSW            DOUBLE    Array[512]
;            FPMW            DOUBLE    Array[512]
;            FPLW            DOUBLE    Array[512]
;            nu_ref_psw      DOUBLE    double       reference frequency PSW
;            nu_ref_pmw      DOUBLE    double       reference frequency PMW
;            nu_ref_plw      DOUBLE    double       reference frequency PLW
;
;    alpha   index of powerlaw source spectrum F_nu = c*nu^alpha where c is an a\
;            arbitrary constant and nu is frequency.
;   
;    pixsize integer pixel size in arcsec
;
;    npix    integer number of pixels along one side of quadratic output array
;
;    pos     double array[2] x/y source position in pixel coordinates
;
; Output:
;    arrout  double array[npix,npix]  output array with 1 Jy point source 
; 
;=========================================================================================
;-

function hs_broadbpsfpix, det, str, fstr, alpha, pixsize, npix, pos

  nnpix = npix * pixsize
  ppos = pos * pixsize
  
  arrout = dblarr(nnpix,nnpix)

  dist_circle, circle, [nnpix,nnpix], ppos[0], ppos[1], /double
  
  hs_broadbpsf, det, str, fstr, alpha, nu, transm, bbpsf
  rad = indgen(n_elements(bbpsf))
  
  for i=0, nnpix-1 do  $
    arrout[i,*] = interpol(bbpsf,rad, reform(circle[i,*]))

  arrout = frebin( arrout, npix, npix)
  return, arrout
  end
  
;+
;=========================================================================================
; Test SPIRE Broaddband Beam Profile Model
; Example how to use the SPIRE broadband beam profile model
;
; Input:    none
;
; Output:   plots and printout
;
;=========================================================================================
;-

pro test_hs_broadbpsf

    hs_getunits, unit, const

;-----------------------------------------------------------------------------------------
; Read beam profile elements and establish radius vector

    hs_rdradpsf, str
    nrad = n_elements(str.pswcore)      ; number of elements in beam profile file
    rad = indgen(nrad)                  ; Make radius vector. The radii are spaced 
                                        ; in 1 arcsec steps
    
;-----------------------------------------------------------------------------------------
; Read filter profiles and aperture efficiencies

    hs_rdfilt, fstr

;-----------------------------------------------------------------------------------------
; Display some bradband radial beam profiles

;Neptune spectral indices
    alphanep_psw = 1.29
    alphanep_pmw = 1.42
    alphanep_plw = 1.47

; Neptune spectrum
    hs_broadbpsf, 'PSW', str, fstr, alphanep_psw, nu, transm_psw, bbpsf_psw
    hs_broadbpsf, 'PMW', str, fstr, alphanep_pmw, nu, transm_pmw, bbpsf_pmw
    hs_broadbpsf, 'PLW', str, fstr, alphanep_plw, nu, transm_plw, bbpsf_plw

; alpha = -1 spectrum
    hs_broadbpsf, 'PSW', str, fstr, -1, nu, transmstd_psw, bbpsfstd_psw
    hs_broadbpsf, 'PMW', str, fstr, -1, nu, transmstd_pmw, bbpsfstd_pmw
    hs_broadbpsf, 'PLW', str, fstr, -1, nu, transmstd_plw, bbpsfstd_plw

    plot_io, rad, bbpsf_psw, $
            ystyle=1, xstyle=1, yrange=[1e-10,1], xrange=[0,1400], $
            title = 'Radialized Beam Profile Model for the three SPIRE bands', $
            xtitle='Radius [arcsec]', ytitle='Normalized intensity', charsize = 1.8, $
            xthick=1.5, ythick=1.5
    oplot, rad, bbpsfstd_psw, linestyle=1
    oplot, rad, bbpsf_pmw
    oplot, rad, bbpsfstd_pmw, linestyle=1
    oplot, rad, bbpsf_plw
    oplot, rad, bbpsfstd_plw, linestyle=1
    al_legend, linestyle=[0,1], ["Neptune spectrum", "Standard nu*F_nu=const. SED"]


;-----------------------------------------------------------------------------------------
; Calculate Solid Angles for two different spectral indices alpha

    form0 = '(a13,2x,a6  ,x,a6  ,x,a6  )'
    form1 = '(a13,2x,f6.0,x,f6.0,x,f6.0)'

    sedname = ['Neptune','alpha=-1 SED']
    alphalst = [[alphanep_psw,alphanep_pmw,alphanep_plw],[-1.0,-1.0,-1.0]]
    print, form=form0, 'SED','PSW','PMW','PLW'
    
    ni = n_elements(alphalst[0,*])
    for i=0, ni-1 do begin      ;spectral index of source spectrum
      hs_broadbpsf, 'PSW', str, fstr, alphalst[0,i], nu, tpsw, bbpswpsf
      hs_broadbpsf, 'PMW', str, fstr, alphalst[1,i], nu, tpmw, bbpmwpsf
      hs_broadbpsf, 'PLW', str, fstr, alphalst[2,i], nu, tplw, bbplwpsf

      bbpswpsf = bbpswpsf/max(bbpswpsf)
      bbpmwpsf = bbpmwpsf/max(bbpmwpsf)
      bbplwpsf = bbplwpsf/max(bbplwpsf)


      omega_psw = total(bbpswpsf * rad*2*!PI)
      omega_pmw = total(bbpmwpsf * rad*2*!PI)
      omega_plw = total(bbplwpsf * rad*2*!PI)

      print, form=form1, sedname[i], omega_psw, omega_pmw, omega_plw
    endfor


;-----------------------------------------------------------------------------------------
; Create map with sources of different color

    
    margin = 20
    alphas = indgen(9)-4.
    nalpha = n_elements(alphas)

    nx = 5  ;number of columns to draw
    
    dets = ['PSW','PMW','PLW']
    pixsizes = [6.,10.,14.]
    ndet = n_elements(dets)
    
    for idet=0, ndet-1 do begin

      det = dets[idet]
      pixsize = pixsizes[idet]

      npix = 31


      ydim = margin*2 + (n_elements(alphas)-0)*npix*2.
      xdim = margin*2 + (nx-0)*npix*2
    
      mapout = dblarr(xdim,ydim)

      for ialpha = 0, nalpha-1 do begin
        for ix = 0, nx-1 do begin
          pxoffs = ix*1.0/nx                    ;offset from center pixel
          xpos = margin + ix*2*npix
          ypos = margin + ialpha*npix*2
          alpha = alphas[ialpha]
          rpos = [npix/2., npix/2.+pxoffs]
          ;print, det, alpha, pixsize, npix, rpos[0], rpos[1], pxoffs
          mapout[xpos:xpos+npix-1,ypos:ypos+npix-1] = $
            hs_broadbpsfpix(det, str, fstr, alpha, pixsize, npix, rpos)
        
        endfor
      endfor

      mkhdr, hd, mapout
      make_astr, astr, crpix=[1,1], crval=[60.0,10.0], delt = [-1.0D, 1.0D]*pixsize/3600.
      putast, hd, astr, equinox = 2000.0
      fxaddpar, hd, 'DETECTOR', det, 'SPIRE Detector name'
      fxaddpar, hd, 'HISTORY', 'Simulation of a 1Jy point source with different colors and positions'

      filename = 'point_src_sim_'+det+'.fits'
      writefits, filename, mapout, hd, /compress
      print ,'Wrote file: ' ,filename
    endfor
    
end



;+
;=========================================================================================
; Define here the path to the additional data files:
;
; SpirePhotRSRF.fits
; PLW_MCore_10.csv    
; PMW_MCore_10.csv                 
; PSW_MCore_10.csv

defsysv, '!hs_datapath', $
  "/Users/bschulz/Documents/spire/calibration/psf/PsfSynthesis4/"
end
